/* extension.js
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * SPDX-License-Identifier: GPL-2.0-or-later
 */

/* exported init */


'use strict';

const { GObject, Shell, St } = imports.gi;

const Main = imports.ui.main;
const PanelMenu = imports.ui.panelMenu;
const PopupMenu = imports.ui.popupMenu;
const SystemActions = imports.misc.systemActions;
const ExtensionUtils = imports.misc.extensionUtils;
const System = Main.panel.statusArea.aggregateMenu._system;
const SystemMenu = System.menu;

const GnomeSession = imports.misc.gnomeSession;
const SessionManager = GnomeSession.SessionManager();

const Config = imports.misc.config;
const SHELL_MAJOR_VERSION = parseInt(Config.PACKAGE_VERSION.split('.')[0]);

let DefaultActions;
let separator1 = new PopupMenu.PopupSeparatorMenuItem;
let separator2 = new PopupMenu.PopupSeparatorMenuItem;
let restart;
let power;
let logout;
let suspend;
let switchUser;

var _bringOut = new GObject.registerClass(
class BringOutSubmenu extends PanelMenu.SystemIndicator {

_init() {
	DefaultActions = new SystemActions.getDefault();

	this._settings = ExtensionUtils.getSettings();
	
	SystemMenu.actor.remove_child(System._sessionSubMenu);
	
	this._createMenu();
	this._connectSettings();
	this._takeAction();
	
	SystemMenu.connect('open-state-changed', (menu, open) => {
		if(!open)
		return;
		DefaultActions.forceUpdate();
		});
	}

_createMenu() {
	let bindFlags = GObject.BindingFlags.DEFAULT | GObject.BindingFlags.SYNC_CREATE;
	let forceLockDown = this._settings.get_boolean('force-lock-down');
	let boolean;
	
	// Separator1
	
	boolean = this._settings.get_boolean('remove-separator-1');
	if (!boolean) { SystemMenu.addMenuItem(separator1); };
	
	// Suspend

	suspend = new PopupMenu.PopupImageMenuItem(_('Suspend'), 'media-playback-pause-symbolic');
	suspend.connect('activate', () => {
						DefaultActions.activateSuspend();
				});
	
	if(!forceLockDown) {
			SystemMenu.addMenuItem(suspend);
			DefaultActions.bind_property('can-suspend', suspend, 'visible', bindFlags);
			} else {
				boolean = this._settings.get_boolean('remove-suspend-button');
				if (!boolean) {
				SystemMenu.addMenuItem(suspend);
				// DefaultActions.bind_property('can-suspend', suspend, 'visible', bindFlags);
						}
				}
				
	// Restart

	restart = new PopupMenu.PopupImageMenuItem(_('Restart…'), 'system-reboot-symbolic');
	restart.connect('activate', () => {
	        				SHELL_MAJOR_VERSION >= 40 ? DefaultActions.activateRestart() : SessionManager.RebootRemote();
        			});
	
	if(!forceLockDown) {
			SystemMenu.addMenuItem(restart);
			SHELL_MAJOR_VERSION >= 40 ? DefaultActions.bind_property('can-restart', restart, 'visible', bindFlags) :
							DefaultActions.bind_property('can-power-off', restart, 'visible', bindFlags);
			} else {
				boolean = this._settings.get_boolean('remove-restart-button');
				if (!boolean) {
				SystemMenu.addMenuItem(restart);
				// SHELL_MAJOR_VERSION >= 40 ? DefaultActions.bind_property('can-restart', this._restartButton, 'visible', bindFlags) :
								// DefaultActions.bind_property('can-power-off', this._restartButton, 'visible', bindFlags);
						}
				}
				
	// Power

	power = new PopupMenu.PopupImageMenuItem(_('Power Off…'), 'system-shutdown-symbolic');
	power.connect('activate', () => { DefaultActions.activatePowerOff(); });
	
	if(!forceLockDown) {
			SystemMenu.addMenuItem(power);
			DefaultActions.bind_property('can-power-off', power, 'visible', bindFlags);
			} else {
				boolean = this._settings.get_boolean('remove-power-button');
				if (!boolean) {
				SystemMenu.addMenuItem(power);
				// DefaultActions.bind_property('can-suspend', suspend, 'visible', bindFlags);
						}
				}
				
	// Separator2
	
	boolean = this._settings.get_boolean('remove-separator-2');
	if (!boolean) { SystemMenu.addMenuItem(separator2); };
	
	// Logout

	logout = new PopupMenu.PopupImageMenuItem(_('Log Out'), 'system-log-out-symbolic');
	logout.connect('activate', () => { DefaultActions.activateLogout(); });
	
	if(!forceLockDown) {
			SystemMenu.addMenuItem(logout);
			DefaultActions.bind_property('can-logout', logout, 'visible', bindFlags);
			} else {
				boolean = this._settings.get_boolean('remove-logout-button');
				if (!boolean) {
				SystemMenu.addMenuItem(logout);
				// DefaultActions.bind_property('can-logout', logout, 'visible', bindFlags);
						}
				}
				
	// Switch User

	switchUser = new PopupMenu.PopupImageMenuItem(_('Switch User…'), 'system-switch-user-symbolic');
	SystemMenu.addMenuItem(switchUser)
	switchUser.connect('activate', () => { DefaultActions.activatSwitchUser(); });
	DefaultActions.bind_property('can-switch-user', switchUser, 'visible', bindFlags);
	
	DefaultActions.forceUpdate();	

	}
	
_connectSettings() {
	this.forceLockDownChanged = this._settings.connect('changed::force-lock-down', this._takeAction.bind(this));
	this.removeSeparator1Changed = this._settings.connect('changed::remove-separator-1', this._takeAction.bind(this));
        this.removeSuspendButtonChanged = this._settings.connect('changed::remove-suspend-button', this._takeAction.bind(this));
        this.removeRestartButtonChanged = this._settings.connect('changed::remove-restart-button', this._takeAction.bind(this));
        this.removePoweroffButtonChanged = this._settings.connect('changed::remove-power-button', this._takeAction.bind(this));
        this.removeSeparator2Changed = this._settings.connect('changed::remove-separator-2', this._takeAction.bind(this));
        this.removeLogoutButtonChanged = this._settings.connect('changed::remove-logout-button', this._takeAction.bind(this));
	}

_onDestroy() {
	
	if(this.forceLockDownChanged) {
	this._settings.disconnect(this.forceLockDownChanged);
	this.forceLockDownChanged = 0;
	}
	
	if(this.removeSeparator1Changed) {
	this._settings.disconnect(this.removeSeparator1Changed);
	this.removeSeparator1Changed = 0;
	}

	if(this.removeSuspendButtonChanged) {
        this._settings.disconnect(this.removeSuspendButtonChanged);
        this.removeSuspendButtonChanged = 0;
        }

        if(this.removeRestartButtonChanged) {
        this._settings.disconnect(this.removeRestartButtonChanged);
        this.removeRestartButtonChanged = 0;
        }
        
        if(this.removePoweroffButtonChanged) {
        this._settings.disconnect(this.removePoweroffButtonChanged);
        this.removePoweroffButtonChanged = 0;
        }
        
        if(this.removeSeparator2Changed) {
	this._settings.disconnect(this.removeSeparator2Changed);
	this.removeSeparator2Changed = 0;
	}

	if(this.removeLogoutButtonChanged) {
        this._settings.disconnect(this.removeLogoutButtonChanged);
        this.removeLogoutButtonChanged = 0;
        }
        
	}
	
_removeActors() {
	SystemMenu.box.remove_actor(separator1);
	SystemMenu.box.remove_actor(suspend);
	SystemMenu.box.remove_actor(restart);
	SystemMenu.box.remove_actor(power);
	SystemMenu.box.remove_actor(separator2);
	SystemMenu.box.remove_actor(logout);
	SystemMenu.box.remove_actor(switchUser);
	}
	
_takeAction() {
	this._removeActors();
	this._createMenu();
	}
});

function init() {
}

let modifiedMenu;

function enable() {
modifiedMenu = new _bringOut();
}

function disable() {
	modifiedMenu._removeActors();
	modifiedMenu._onDestroy();
	modifiedMenu = null;
	SystemMenu.box.insert_child_at_index(System._sessionSubMenu, SystemMenu.numMenuItems);
}
