const GLib = imports.gi.GLib;
const Gtk = imports.gi.Gtk;
const Gdk = imports.gi.Gdk;

const ExtensionUtils = imports.misc.extensionUtils;

const GTK_VERSION = Gtk.get_major_version();

function init() {
}

function buildPrefsWidget() {
    let widget = new PrefsWidget();
    	if(GTK_VERSION == 3) {
    		widget.widget.show_all(); }
    		
    return widget.widget;
}

class PrefsWidget {
    constructor() {
        this._settings = ExtensionUtils.getSettings();

        this.widget = new Gtk.Box({
            orientation: Gtk.Orientation.VERTICAL,
        });

        this.vbox = new Gtk.Box({
            orientation: Gtk.Orientation.VERTICAL,
            hexpand: true,
        });
        
        if(GTK_VERSION == 3) {
        	this.vbox.add(this.forceLockDown());
        	this.vbox.add(this.removeSeparator1());
        	this.vbox.add(this.removeSuspendButton());
        	this.vbox.add(this.removeRestartButton());
        	this.vbox.add(this.removePoweroffButton());
        	this.vbox.add(this.removeSeparator2());
        	this.vbox.add(this.removeLogoutButton());
        	this.vbox.add(this.lockdownTip());
        	
        	this.widget.add(this.vbox);        
        }
        
        else if(GTK_VERSION == 4) {
        	this.vbox.append(this.forceLockDown());
        	this.vbox.append(this.removeSeparator1());
        	this.vbox.append(this.removeSuspendButton());
        	this.vbox.append(this.removeRestartButton());
        	this.vbox.append(this.removePoweroffButton());
        	this.vbox.append(this.removeSeparator2());
        	this.vbox.append(this.removeLogoutButton());
        	this.vbox.append(this.lockdownTip());
        	
        	this.widget.append(this.vbox);
    	}
    }
    
forceLockDown() {
    	let hbox = new Gtk.Box({ orientation: Gtk.Orientation.HORIZONTAL, margin_top: 5, halign: Gtk.Align.CENTER   });
    	let forceLockDownLabel = new Gtk.Label({ label: "<Force Lockdown Keys> - Turn ON to Control Below 4 Action Buttons      ", xalign: 0, hexpand: true });
    	
        this.fLDToggleSwitch = new Gtk.Switch({ active: this._settings.get_boolean('force-lock-down') });
        this.fLDToggleSwitch.connect('notify::active', (button) => { this._settings.set_boolean('force-lock-down', button.active); }); 
    	
    	if(GTK_VERSION == 3) {
		hbox.add(forceLockDownLabel);
		hbox.add(this.fLDToggleSwitch) }
		
        else if(GTK_VERSION == 4) {
		hbox.append(forceLockDownLabel);
		hbox.append(this.fLDToggleSwitch) }
      	
      	return hbox;
    } 
    
removeSeparator1() {
    	let hbox = new Gtk.Box({ orientation: Gtk.Orientation.HORIZONTAL, margin_top: 5  });
    	let removeSeparator1Label = new Gtk.Label({ label: "Remove Separator 1", xalign: 0, hexpand: true });
    	
        this.rS1ToggleSwitch = new Gtk.Switch({ active: this._settings.get_boolean('remove-separator-1') });
        this.rS1ToggleSwitch.connect('notify::active', (button) => { this._settings.set_boolean('remove-separator-1', button.active); }); 
    	
    	if(GTK_VERSION == 3) {
		hbox.add(removeSeparator1Label);
		hbox.add(this.rS1ToggleSwitch) }
		
        else if(GTK_VERSION == 4) {
		hbox.append(removeSeparator1Label);
		hbox.append(this.rS1ToggleSwitch) }
      	
      	return hbox;
    } 
    
removeSuspendButton() {
    	let hbox = new Gtk.Box({ orientation: Gtk.Orientation.HORIZONTAL, margin_top: 5  });
    	let removeSuspendButtonLabel = new Gtk.Label({ label: "Remove Suspend Button (Action Button)", xalign: 0, hexpand: true });
    	
        this.rSBLToggleSwitch = new Gtk.Switch({ active: this._settings.get_boolean('remove-suspend-button') });
        this.rSBLToggleSwitch.connect('notify::active', (button) => { this._settings.set_boolean('remove-suspend-button', button.active); }); 
    	
    	if(GTK_VERSION == 3) {
		hbox.add(removeSuspendButtonLabel);
		hbox.add(this.rSBLToggleSwitch) }
		
        else if(GTK_VERSION == 4) {
		hbox.append(removeSuspendButtonLabel);
		hbox.append(this.rSBLToggleSwitch) }
      	
      	return hbox;
    } 
    
removeRestartButton() {
    	let hbox = new Gtk.Box({ orientation: Gtk.Orientation.HORIZONTAL, margin_top: 5 });
    	let removeRestartButtonLabel = new Gtk.Label({ label: "Remove Restart Button (Action Button)", xalign: 0, hexpand: true });
    	
        this.rRBLToggleSwitch = new Gtk.Switch({ active: this._settings.get_boolean('remove-restart-button') });
        this.rRBLToggleSwitch.connect('notify::active', (button) => { this._settings.set_boolean('remove-restart-button', button.active); }); 
    	
    	if(GTK_VERSION == 3) {
		hbox.add(removeRestartButtonLabel);
		hbox.add(this.rRBLToggleSwitch) }
		
        else if(GTK_VERSION == 4) {
		hbox.append(removeRestartButtonLabel);
		hbox.append(this.rRBLToggleSwitch) }
      	
      	return hbox;
    } 

removePoweroffButton() {
    	let hbox = new Gtk.Box({ orientation: Gtk.Orientation.HORIZONTAL, margin_top: 5 });
    	let removePoweroffButtonLabel = new Gtk.Label({ label: "Remove Poweroff Button (Action Button)", xalign: 0, hexpand: true });
    	
        this.rPBLToggleSwitch = new Gtk.Switch({ active: this._settings.get_boolean('remove-power-button') });
        this.rPBLToggleSwitch.connect('notify::active', (button) => { this._settings.set_boolean('remove-power-button', button.active); }); 
    	
    	if(GTK_VERSION == 3) {
		hbox.add(removePoweroffButtonLabel);
		hbox.add(this.rPBLToggleSwitch) }
		
        else if(GTK_VERSION == 4) {
		hbox.append(removePoweroffButtonLabel);
		hbox.append(this.rPBLToggleSwitch) }
      	
      	return hbox;
    }
    
removeSeparator2() {
    	let hbox = new Gtk.Box({ orientation: Gtk.Orientation.HORIZONTAL, margin_top: 5  });
    	let removeSeparator2Label = new Gtk.Label({ label: "Remove Separator 2", xalign: 0, hexpand: true });
    	
        this.rS2ToggleSwitch = new Gtk.Switch({ active: this._settings.get_boolean('remove-separator-2') });
        this.rS2ToggleSwitch.connect('notify::active', (button) => { this._settings.set_boolean('remove-separator-2', button.active); }); 
    	
    	if(GTK_VERSION == 3) {
		hbox.add(removeSeparator2Label);
		hbox.add(this.rS2ToggleSwitch) }
		
        else if(GTK_VERSION == 4) {
		hbox.append(removeSeparator2Label);
		hbox.append(this.rS2ToggleSwitch) }
      	
      	return hbox;
    } 
removeLogoutButton() {
    	let hbox = new Gtk.Box({ orientation: Gtk.Orientation.HORIZONTAL, margin_top: 5 });
    	let removeLogoutButtonLabel = new Gtk.Label({ label: "Remove Logout Button (Action Button)", xalign: 0, hexpand: true });
    	
        this.rLBLToggleSwitch = new Gtk.Switch({ active: this._settings.get_boolean('remove-logout-button') });
        this.rLBLToggleSwitch.connect('notify::active', (button) => { this._settings.set_boolean('remove-logout-button', button.active); }); 
    	
    	if(GTK_VERSION == 3) {
		hbox.add(removeLogoutButtonLabel);
		hbox.add(this.rLBLToggleSwitch) }
		
        else if(GTK_VERSION == 4) {
		hbox.append(removeLogoutButtonLabel);
		hbox.append(this.rLBLToggleSwitch) }
      	
      	return hbox;
    }
    
lockdownTip() {
    	let hbox = new Gtk.Box({ orientation: Gtk.Orientation.HORIZONTAL, margin_top: 5, halign: Gtk.Align.START  });
    	
    	let lockdownTipLabel = new Gtk.Label({
    		xalign: 0,
    		useMarkup: true,
    		wrap: true,
    		label: '<b>This below note is applicable only if "Force Lockdown Keys" switch above is Turned ON</b>\n\
if your systems gsettings schema org.gnome.desktop.lockdown with key disable-log-out is set to true(default is false) then\n\
LOGOUT, RESTART, POWEROFF will not take any Action when you click on them. Also in Special Case the\n\
SUSPEND button will not take any Action.'
    		});
    	
    	if(GTK_VERSION == 3) {
		hbox.add(lockdownTipLabel); }
	else if(GTK_VERSION == 4) {
		hbox.append(lockdownTipLabel); }
		
        return hbox;
    }  
}




